﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PicoScopeTestApp
{
    class Program
    {
        private static List<short> ApplicationBuffer = new List<short>();
        private static uint number_of_total_values = 0;
        private static uint number_of_new_values = 0;
        private static short ScopeHandle = 0;

        //Set this variable to 1 if you want to see the program crash, or 2 if you want to see it not crash.
        private static int program_version = 1;

        private enum PS2000_INFO_ENUM
        {
            PS2000_DRIVER_VERSION,
            PS2000_USB_VERSION,
            PS2000_HARDWARE_VERSION,
            PS2000_VARIANT_INFO,
            PS2000_BATCH_AND_SERIAL,
            PS2000_CAL_DATE,
            PS2000_ERROR_CODE,
            PS2000_KERNEL_DRIVER_VERSION
        }
        
        public static string GetScopeInformation (short scopeHandle, short info_type)
        {
            StringBuilder returned_info = new StringBuilder(80);
            Imports.GetUnitInfo(scopeHandle, returned_info, 80, info_type);
            return returned_info.ToString();
        }

        private static unsafe void get_overview_buffers(short** overviewbuffers, short overflow, uint triggeredAt, short triggered, short auto_stop,
        uint nValues)
        {
            for (int x = 0; x < nValues; x++)
            {
                ApplicationBuffer.Add(overviewbuffers[0][x]);
            }

            number_of_total_values += nValues;
            number_of_new_values = nValues;
        }

        public static void Main(string[] args)
        {
            //Open the scope
            ScopeHandle = Imports.OpenUnit();

            //Output stats about the scope and the driver
            string driver_version = GetScopeInformation(ScopeHandle, Convert.ToInt16(PS2000_INFO_ENUM.PS2000_DRIVER_VERSION));
            string usb_version = GetScopeInformation(ScopeHandle, Convert.ToInt16(PS2000_INFO_ENUM.PS2000_USB_VERSION));
            string hardware_version = GetScopeInformation(ScopeHandle, Convert.ToInt16(PS2000_INFO_ENUM.PS2000_HARDWARE_VERSION));
            string variant_info = GetScopeInformation(ScopeHandle, Convert.ToInt16(PS2000_INFO_ENUM.PS2000_VARIANT_INFO));
            string serial_info = GetScopeInformation(ScopeHandle, Convert.ToInt16(PS2000_INFO_ENUM.PS2000_BATCH_AND_SERIAL));
            string cal_date = GetScopeInformation(ScopeHandle, Convert.ToInt16(PS2000_INFO_ENUM.PS2000_CAL_DATE));
            string error_codes = GetScopeInformation(ScopeHandle, Convert.ToInt16(PS2000_INFO_ENUM.PS2000_ERROR_CODE));
            string kernel_version = GetScopeInformation(ScopeHandle, Convert.ToInt16(PS2000_INFO_ENUM.PS2000_KERNEL_DRIVER_VERSION));

            System.Console.WriteLine("Driver version: " + driver_version);
            System.Console.WriteLine("USB version: " + usb_version);
            System.Console.WriteLine("Hardware version: " + hardware_version);
            System.Console.WriteLine("Variant info: " + variant_info);
            System.Console.WriteLine("Serial info: " + serial_info);
            System.Console.WriteLine("Calibrated date: " + cal_date);
            System.Console.WriteLine("Error code: " + error_codes);
            System.Console.WriteLine("Kernel version: " + kernel_version);

            //Set up channel A
            short channel_successful = Imports.SetChannel(ScopeHandle, Imports.Channel.ChannelA,
                1, 1, Imports.Range.Range_20V);
            if (channel_successful == 0)
            {
                throw new Exception("Channel A not working!!!");
            }

            //Set up channel B
            short channel_successful_2 = Imports.SetChannel(ScopeHandle, Imports.Channel.ChannelB,
                1, 1, Imports.Range.Range_20V);
            if (channel_successful_2 == 0)
            {
                throw new Exception("Channel B not working!!!");
            }

            bool start_new_streaming = false;

            //Message to the user about streaming from the scope
            System.Console.WriteLine("Press any key to begin streaming from the oscilloscope.");
            System.Console.WriteLine("After you begin streaming, press any key to quit streaming and exit the program.");

            //Wait for the user to press a key to start streaming.
            System.Console.ReadKey();

            //Tell the user that streaming is beginning.
            System.Console.WriteLine("Beginning streaming...");

            //Set up streaming
            short auto_stop = 0;
            uint microseconds_per_sample = 10;
            uint buffer_size = 15000;
            short result = Imports.ps2000_run_streaming_ns(ScopeHandle, microseconds_per_sample,
                Imports.ReportedTimeUnits.MicroSeconds,
                buffer_size, auto_stop, 1, buffer_size);
            if (result == 0)
            {
                throw new Exception("Streaming not enabled!!!!");
            }
            
            //Stream until another key press occurs.
            while (!Console.KeyAvailable)
            {
                if (program_version == 2 && start_new_streaming)
                {
                    result = Imports.ps2000_run_streaming_ns(ScopeHandle, microseconds_per_sample,
                        Imports.ReportedTimeUnits.MicroSeconds,
                        buffer_size, auto_stop, 1, buffer_size);
                    if (result == 0)
                    {
                        throw new Exception("Streaming not enabled!!!!");
                    }

                    start_new_streaming = false;
                }

                //Read new data from the oscilloscope
                unsafe
                {
                    //Grab values from the oscilloscope and put them into our ChannelA and ChannelB arrays
                    Imports.ps2000_get_streaming_last_values(ScopeHandle, get_overview_buffers);
                }

                //Print the total number of values to the screen
                System.Console.WriteLine(number_of_total_values.ToString());

                //Reset the application buffer for the next call into the get_overview_buffers function
                ApplicationBuffer.Clear();

                if (program_version == 2 && number_of_total_values >= buffer_size)
                {
                    Imports.Stop(ScopeHandle);
                    number_of_total_values = 0;
                    start_new_streaming = true;
                }
            }
            
            //Close the oscilloscope
            Imports.CloseUnit(ScopeHandle);
        }
    }
}
